@@ -1,4 +1,4 @@
-# Copyright 2019-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -33,6 +33,7 @@ Code details
 """
 
 # pylint: disable=invalid-name
+from enum import Enum, auto
 from typing import FrozenSet, Iterable, List, Optional, Sequence, Union
 
 from braket.aws import AwsDevice, AwsDeviceType, AwsQuantumTask, AwsQuantumTaskBatch, AwsSession
@@ -41,10 +42,17 @@ from braket.device_schema import DeviceActionType
 from braket.devices import Device, LocalSimulator
 from braket.simulator import BraketSimulator
 from braket.tasks import GateModelQuantumTaskResult, QuantumTask
-from pennylane import CircuitGraph, QubitDevice
+from pennylane import CircuitGraph, QuantumFunctionError, QubitDevice
 from pennylane import numpy as np
-from pennylane.operation import Expectation, Observable, Operation, Probability, Sample, Variance
-from pennylane.qnodes import QuantumFunctionError
+from pennylane.operation import (
+    Expectation,
+    Observable,
+    Operation,
+    Probability,
+    Sample,
+    State,
+    Variance,
+)
 
 from braket.pennylane_plugin.translation import (
     supported_operations,
@@ -54,7 +62,13 @@ from braket.pennylane_plugin.translation import (
 
 from ._version import __version__
 
-RETURN_TYPES = [Expectation, Variance, Sample, Probability]
+RETURN_TYPES = [Expectation, Variance, Sample, Probability, State]
+
+
+class Shots(Enum):
+    """Used to specify the default number of shots in BraketAwsQubitDevice"""
+
+    DEFAULT = auto()
 
 
 class BraketQubitDevice(QubitDevice):
@@ -65,14 +79,13 @@ class BraketQubitDevice(QubitDevice):
             or iterable that contains unique labels for the subsystems as numbers
             (i.e., ``[-1, 0, 2]``) or strings (``['ancilla', 'q1', 'q2']``).
         device (Device): The Amazon Braket device to use with PennyLane.
-        shots (int): Number of circuit evaluations or random samples included,
-            to estimate expectation values of observables. If this value is set to 0,
-            the device runs in analytic mode (calculations will be exact).
-            The device's ``shots`` property is set to 1 and ignored.
+        shots (int or None): Number of circuit evaluations or random samples included,
+            to estimate expectation values of observables. If this value is set to ``None`` or
+            ``0``, the device runs in analytic mode (calculations will be exact).
         **run_kwargs: Variable length keyword arguments for ``braket.devices.Device.run()`.
     """
     name = "Braket PennyLane plugin"
-    pennylane_requires = ">=0.11.0"
+    pennylane_requires = ">=0.16.0"
     version = __version__
     author = "Amazon Web Services"
 
@@ -81,15 +94,16 @@ class BraketQubitDevice(QubitDevice):
         wires: Union[int, Iterable],
         device: Device,
         *,
-        shots: int,
+        shots: Union[int, None],
         **run_kwargs,
     ):
-        # `shots` cannot be set to 0, but is ignored anyways
-        super().__init__(wires, shots or 1, analytic=shots == 0)
+        super().__init__(wires, shots=shots or None)
         self._device = device
         self._circuit = None
         self._task = None
         self._run_kwargs = run_kwargs
+        self._supported_ops = supported_operations(self._device)
+        self._check_supported_result_types()
 
     def reset(self):
         super().reset()
@@ -106,7 +120,7 @@ class BraketQubitDevice(QubitDevice):
     @property
     def operations(self) -> FrozenSet[str]:
         """FrozenSet[str]: The set of names of PennyLane operations that the device supports."""
-        return supported_operations()
+        return self._supported_ops
 
     @property
     def circuit(self) -> Circuit:
@@ -127,7 +141,9 @@ class BraketQubitDevice(QubitDevice):
         )
         for observable in circuit.observables:
             dev_wires = self.map_wires(observable.wires).tolist()
-            braket_circuit.add_result_type(translate_result_type(observable, dev_wires))
+            braket_circuit.add_result_type(
+                translate_result_type(observable, dev_wires, self._braket_result_types)
+            )
         return braket_circuit
 
     def statistics(
@@ -184,8 +200,7 @@ class BraketQubitDevice(QubitDevice):
 
         # Add operations to Braket Circuit object
         for operation in operations + rotations:
-            params = [p.numpy() if isinstance(p, np.tensor) else p for p in operation.parameters]
-            gate = translate_operation(operation, params)
+            gate = translate_operation(operation)
             dev_wires = self.map_wires(operation.wires).tolist()
             ins = Instruction(gate, dev_wires)
             circuit.add_instruction(ins)
@@ -198,12 +213,23 @@ class BraketQubitDevice(QubitDevice):
 
         return circuit
 
+    def _check_supported_result_types(self):
+        supported_result_types = self._device.properties.action[
+            "braket.ir.jaqcd.program"
+        ].supportedResultTypes
+
+        self._braket_result_types = frozenset(
+            result_type.name for result_type in supported_result_types
+        )
+
     def _run_task(self, circuit):
         raise NotImplementedError("Need to implement task runner")
 
     def _get_statistic(self, braket_result, observable):
         dev_wires = self.map_wires(observable.wires).tolist()
-        return braket_result.get_value_by_result_type(translate_result_type(observable, dev_wires))
+        return braket_result.get_value_by_result_type(
+            translate_result_type(observable, dev_wires, self._braket_result_types)
+        )
 
 
 class BraketAwsQubitDevice(BraketQubitDevice):
@@ -222,13 +248,12 @@ class BraketAwsQubitDevice(BraketQubitDevice):
         poll_timeout_seconds (float): Total time in seconds to wait for
             results before timing out.
         poll_interval_seconds (float): The polling interval for results in seconds.
-        shots (int): Number of circuit evaluations or random samples included,
-            to estimate expectation values of observables. If this value is set to 0
-            and the device ARN points to a simulator, the device runs
-            in analytic mode (calculations will be exact). The device's
-            ``shots`` property is set to 1 and ignored; trying to use 0 shots
-            with QPUs will fail.
-            Defaults to 1000 for QPUs and 0 for simulators.
+        shots (int, None or Shots.DEFAULT): Number of circuit evaluations or random samples
+            included, to estimate expectation values of observables. If set to Shots.DEFAULT,
+            uses the default number of shots specified by the remote device. If ``shots`` is set
+            to ``0`` or ``None``, the device runs in analytic mode (calculations will be exact).
+            Analytic mode is not available on QPU and hence an error will be raised.
+            Default: Shots.DEFAULT
         aws_session (Optional[AwsSession]): An AwsSession object created to manage
             interactions with AWS services, to be supplied if extra control
             is desired. Default: None
@@ -255,7 +280,7 @@ class BraketAwsQubitDevice(BraketQubitDevice):
         device_arn: str,
         s3_destination_folder: AwsSession.S3DestinationFolder,
         *,
-        shots: Optional[int] = None,
+        shots: Union[int, None, Shots] = Shots.DEFAULT,
         poll_timeout_seconds: float = AwsQuantumTask.DEFAULT_RESULTS_POLL_TIMEOUT,
         poll_interval_seconds: float = AwsQuantumTask.DEFAULT_RESULTS_POLL_INTERVAL,
         aws_session: Optional[AwsSession] = None,
@@ -270,17 +295,18 @@ class BraketAwsQubitDevice(BraketQubitDevice):
             raise ValueError(f"Device {device.name} does not support quantum circuits")
 
         device_type = device.type
-        if shots is not None:
-            if shots == 0 and device_type == AwsDeviceType.QPU:
-                raise ValueError("QPU devices do not support 0 shots")
-            num_shots = shots
-        elif device_type == AwsDeviceType.SIMULATOR:
-            num_shots = AwsDevice.DEFAULT_SHOTS_SIMULATOR
-        elif device_type == AwsDeviceType.QPU:
-            num_shots = AwsDevice.DEFAULT_SHOTS_QPU
-        else:
+        if device_type not in (AwsDeviceType.SIMULATOR, AwsDeviceType.QPU):
             raise ValueError(f"Invalid device type: {device_type}")
 
+        if shots == Shots.DEFAULT and device_type == AwsDeviceType.SIMULATOR:
+            num_shots = AwsDevice.DEFAULT_SHOTS_SIMULATOR
+        elif shots == Shots.DEFAULT and device_type == AwsDeviceType.QPU:
+            num_shots = AwsDevice.DEFAULT_SHOTS_QPU
+        elif (shots is None or shots == 0) and device_type == AwsDeviceType.QPU:
+            raise ValueError("QPU devices do not support 0 shots")
+        else:
+            num_shots = shots
+
         super().__init__(wires, device, shots=num_shots, **run_kwargs)
         self._s3_folder = s3_destination_folder
         self._poll_timeout_seconds = poll_timeout_seconds
@@ -345,11 +371,10 @@ class BraketLocalQubitDevice(BraketQubitDevice):
         backend (Union[str, BraketSimulator]): The name of the simulator backend or
             the actual simulator instance to use for simulation. Defaults to the
             ``default`` simulator backend name.
-        shots (int): Number of circuit evaluations or random samples included,
-            to estimate expectation values of observables. If this value is set to 0,
-            then the device runs in analytic mode (calculations will be exact);
-            the device's ``shots`` property is set to 1 and ignored.
-            Default: 0
+        shots (int or None): Number of circuit evaluations or random samples included,
+            to estimate expectation values of observables. If this value is set to ``None`` or
+            ``0``, then the device runs in analytic mode (calculations will be exact).
+            Default: None
         **run_kwargs: Variable length keyword arguments for ``braket.devices.Device.run()``.
     """
     name = "Braket LocalSimulator for PennyLane"
@@ -360,7 +385,7 @@ class BraketLocalQubitDevice(BraketQubitDevice):
         wires: Union[int, Iterable],
         backend: Union[str, BraketSimulator] = "default",
         *,
-        shots: int = 0,
+        shots: Union[int, None] = None,
         **run_kwargs,
     ):
         device = LocalSimulator(backend)
