@@ -1,4 +1,4 @@
-# Copyright 2019-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -25,79 +25,34 @@ These operations can be imported via
 
 .. code-block:: python
 
-    from braket.pennylane_braket.ops import CPHASE, ISWAP, PSWAP
+    from braket.pennylane_plugin import (
+        PSWAP,
+        XY,
+        YY,
+        CPhaseShift00,
+        CPhaseShift01,
+        CPhaseShift10,
+    )
 
 Operations
 ----------
 
 .. autosummary::
-    CPhaseShift
     CPhaseShift00
     CPhaseShift01
     CPhaseShift10
-    ISWAP
     PSWAP
     XY
-    XX
     YY
-    ZZ
 
 Code details
 ~~~~~~~~~~~~
 """
+
 import numpy as np
 import pennylane as qml
 from pennylane.operation import Operation
-
-
-class CPhaseShift(Operation):
-    r""" CPhaseShift(phi, wires)
-
-    Controlled phase shift gate phasing the :math:`| 11 \rangle` state.
-
-    .. math::
-
-      \mathtt{CPhaseShift}(\phi) = \begin{bmatrix}
-          1 & 0 & 0 & 0 \\
-          0 & 1 & 0 & 0 \\
-          0 & 0 & 1 & 0 \\
-          0 & 0 & 0 & e^{i \phi}
-      \end{bmatrix}.
-
-    **Details:**
-
-    * Number of wires: 2
-    * Number of parameters: 1
-    * Gradient recipe:
-
-    .. math::
-
-        \frac{d}{d \phi} \mathtt{CPhaseShift}(\phi)
-        = \frac{1}{2} \left[ \mathtt{CPhaseShift}(\phi + \pi / 2)
-            + \mathtt{CPhaseShift}(\phi - \pi / 2) \right]
-
-    Args:
-        phi (float): the controlled phase angle
-        wires (int): the subsystem the gate acts on
-    """
-    num_params = 1
-    num_wires = 2
-    par_domain = "R"
-    grad_method = "A"
-
-    @staticmethod
-    def decomposition(phi, wires):
-        return [
-            qml.PhaseShift(phi / 2, wires=[wires[0]]),
-            qml.PhaseShift(phi / 2, wires=[wires[1]]),
-            qml.CNOT(wires=wires),
-            qml.PhaseShift(-phi / 2, wires=[wires[1]]),
-            qml.CNOT(wires=wires),
-        ]
-
-    @classmethod
-    def _matrix(cls, *params):
-        return np.diag([1.0, 1.0, 1.0, np.exp(1.0j * params[0])])
+from pennylane.ops.qubit import four_term_grad_recipe
 
 
 class CPhaseShift00(Operation):
@@ -121,7 +76,7 @@ class CPhaseShift00(Operation):
     .. math::
         \frac{d}{d \phi} \mathtt{CPhaseShift00}(\phi)
         = \frac{1}{2} \left[ \mathtt{CPhaseShift00}(\phi + \pi / 2)
-            + \mathtt{CPhaseShift00}(\phi - \pi / 2) \right]
+            - \mathtt{CPhaseShift00}(\phi - \pi / 2) \right]
 
     Args:
         phi (float): the controlled phase angle
@@ -148,7 +103,7 @@ class CPhaseShift00(Operation):
 
     @classmethod
     def _matrix(cls, *params):
-        return np.diag([np.exp(1.0j * params[0]), 1.0, 1.0, 1.0])
+        return np.diag(np.array([np.exp(1.0j * params[0]), 1.0, 1.0, 1.0], dtype=complex))
 
 
 class CPhaseShift01(Operation):
@@ -172,7 +127,7 @@ class CPhaseShift01(Operation):
     .. math::
         \frac{d}{d \phi} \mathtt{CPhaseShift01}(\phi)
         = \frac{1}{2} \left[ \mathtt{CPhaseShift01}(\phi + \pi / 2)
-            + \mathtt{CPhaseShift01}(\phi - \pi / 2) \right]
+            - \mathtt{CPhaseShift01}(\phi - \pi / 2) \right]
 
     Args:
         phi (float): the controlled phase angle
@@ -197,7 +152,7 @@ class CPhaseShift01(Operation):
 
     @classmethod
     def _matrix(cls, *params):
-        return np.diag([1.0, np.exp(1.0j * params[0]), 1.0, 1.0])
+        return np.diag(np.array([1.0, np.exp(1.0j * params[0]), 1.0, 1.0], dtype=complex))
 
 
 class CPhaseShift10(Operation):
@@ -221,7 +176,7 @@ class CPhaseShift10(Operation):
     .. math::
         \frac{d}{d \phi} \mathtt{CPhaseShift10}(\phi)
         = \frac{1}{2} \left[ \mathtt{CPhaseShift10}(\phi + \pi / 2)
-            + \mathtt{CPhaseShift10}(\phi - \pi / 2) \right]
+            - \mathtt{CPhaseShift10}(\phi - \pi / 2) \right]
 
     Args:
         phi (float): the controlled phase angle
@@ -246,45 +201,7 @@ class CPhaseShift10(Operation):
 
     @classmethod
     def _matrix(cls, *params):
-        return np.diag([1.0, 1.0, np.exp(1.0j * params[0]), 1.0])
-
-
-class ISWAP(Operation):
-    r""" ISWAP(wires)
-
-    ISWAP gate.
-
-    .. math:: \mathtt{ISWAP} = \begin{bmatrix}
-            1 & 0 & 0 & 0 \\
-            0 & 0 & i & 0 \\
-            0 & i & 0 & 0 \\
-            0 & 0 & 0 & 1
-        \end{bmatrix}.
-
-    **Details:**
-
-    * Number of wires: 2
-    * Number of parameters: 0
-
-    Args:
-        wires (int): the subsystem the gate acts on
-    """
-    num_params = 0
-    num_wires = 2
-    par_domain = None
-
-    @staticmethod
-    def decomposition(wires):
-        return [
-            qml.SWAP(wires=wires),
-            qml.S(wires=[wires[0]]),
-            qml.S(wires=[wires[1]]),
-            qml.CZ(wires=wires),
-        ]
-
-    @classmethod
-    def _matrix(cls, *params):
-        return np.diag([1, 1j, 1j, 1])[[0, 2, 1, 3]]
+        return np.diag(np.array([1.0, 1.0, np.exp(1.0j * params[0]), 1.0], dtype=complex))
 
 
 class PSWAP(Operation):
@@ -307,7 +224,7 @@ class PSWAP(Operation):
 
     .. math::
         \frac{d}{d \phi} \mathtt{PSWAP}(\phi)
-        = \frac{1}{2} \left[ \mathtt{PSWAP}(\phi + \pi / 2) + \mathtt{PSWAP}(\phi - \pi / 2) \right]
+        = \frac{1}{2} \left[ \mathtt{PSWAP}(\phi + \pi / 2) - \mathtt{PSWAP}(\phi - \pi / 2) \right]
 
     Args:
         phi (float): the phase angle
@@ -330,7 +247,9 @@ class PSWAP(Operation):
     @classmethod
     def _matrix(cls, *params):
         phi = params[0]
-        return np.diag([1, np.exp(1j * phi), np.exp(1j * phi), 1])[[0, 2, 1, 3]]
+        return np.diag(np.array([1, np.exp(1j * phi), np.exp(1j * phi), 1], dtype=complex))[
+            [0, 2, 1, 3]
+        ]
 
 
 class XY(Operation):
@@ -349,11 +268,19 @@ class XY(Operation):
 
     * Number of wires: 2
     * Number of parameters: 1
-    * Gradient recipe:
+    * Gradient recipe: The XY operator satisfies a four-term parameter-shift rule
+      (see Appendix F, https://arxiv.org/abs/2104.05695):
 
-    .. math::
-        \frac{d}{d \phi} \mathtt{XY}(\phi)
-        = \frac{1}{2} \left[ \mathtt{XY}(\phi + \pi / 2) + \mathtt{XY}(\phi - \pi / 2) \right]
+      .. math::
+          \frac{d}{d \phi} f(XY(\phi))
+          = c_+ \left[ f(XY(\phi + a)) - f(XY(\phi - a)) \right]
+          - c_- \left[ f(XY(\phi + b)) - f(XY(\phi - b)) \right]
+
+      where :math:`f` is an expectation value depending on :math:`XY(\phi)`, and
+
+      - :math:`a = \pi / 2`
+      - :math:`b = 3 \pi / 2`
+      - :math:`c_{\pm} = (\sqrt{2} \pm 1)/{4 \sqrt{2}}`
 
     Args:
         phi (float): the phase angle
@@ -363,6 +290,7 @@ class XY(Operation):
     num_wires = 2
     par_domain = "R"
     grad_method = "A"
+    grad_recipe = four_term_grad_recipe
 
     @staticmethod
     def decomposition(phi, wires):
@@ -391,61 +319,6 @@ class XY(Operation):
         )
 
 
-class XX(Operation):
-    r""" XX(phi, wires)
-
-    Ising XX coupling gate: https://arxiv.org/abs/1707.06356
-
-    .. math:: \mathtt{XX}(\phi) = \begin{bmatrix}
-            \cos(\phi / 2) & 0 & 0 & -i \sin(\phi / 2) \\
-            0 & \cos(\phi / 2) & -i \sin(\phi / 2) & 0 \\
-            0 & -i \sin(\phi / 2) & \cos(\phi / 2) & 0 \\
-            -i \sin(\phi / 2) & 0 & 0 & \cos(\phi / 2)
-        \end{bmatrix}.
-
-    **Details:**
-
-    * Number of wires: 2
-    * Number of parameters: 1
-    * Gradient recipe:
-
-    .. math::
-        \frac{d}{d \phi} \mathtt{XX}(\phi)
-        = \frac{1}{2} \left[ \mathtt{XX}(\phi + \pi / 2) + \mathtt{XX}(\phi - \pi / 2) \right]
-
-    Args:
-        phi (float): the phase angle
-        wires (int): the subsystem the gate acts on
-    """
-    num_params = 1
-    num_wires = 2
-    par_domain = "R"
-    grad_method = "A"
-
-    @staticmethod
-    def decomposition(phi, wires):
-        return [
-            qml.CNOT(wires=wires),
-            qml.RX(phi, wires=[wires[0]]),
-            qml.CNOT(wires=wires),
-        ]
-
-    @classmethod
-    def _matrix(cls, *params):
-        phi = params[0]
-        cos = np.cos(phi / 2)
-        isin = 1.0j * np.sin(phi / 2)
-        return np.array(
-            [
-                [cos, 0.0, 0.0, -isin],
-                [0.0, cos, -isin, 0.0],
-                [0.0, -isin, cos, 0.0],
-                [-isin, 0.0, 0.0, cos],
-            ],
-            dtype=complex,
-        )
-
-
 class YY(Operation):
     r""" YY(phi, wires)
 
@@ -466,7 +339,7 @@ class YY(Operation):
 
     .. math::
         \frac{d}{d \phi} \mathtt{YY}(\phi)
-        = \frac{1}{2} \left[ \mathtt{YY}(\phi + \pi / 2) + \mathtt{YY}(\phi - \pi / 2) \right]
+        = \frac{1}{2} \left[ \mathtt{YY}(\phi + \pi / 2) - \mathtt{YY}(\phi - \pi / 2) \right]
 
     Args:
         phi (float): the phase angle
@@ -499,50 +372,3 @@ class YY(Operation):
             ],
             dtype=complex,
         )
-
-
-class ZZ(Operation):
-    r""" ZZ(phi, wires)
-
-    Ising ZZ coupling gate: https://arxiv.org/abs/1707.06356
-
-    .. math:: \mathtt{ZZ}(\phi) = \begin{bmatrix}
-            e^{-i \phi / 2} & 0 & 0 & 0 \\
-            0 & e^{i \phi / 2} & 0 & 0 \\
-            0 & 0 & e^{i \phi / 2} & 0 \\
-            0 & 0 & 0 & e^{-i \phi / 2}
-        \end{bmatrix}.
-
-    **Details:**
-
-    * Number of wires: 2
-    * Number of parameters: 1
-    * Gradient recipe:
-
-    .. math::
-        \frac{d}{d \phi} \mathtt{ZZ}(\phi)
-        = \frac{1}{2} \left[ \mathtt{ZZ}(\phi + \pi / 2) + \mathtt{ZZ}(\phi - \pi / 2) \right]
-
-    Args:
-        phi (float): the phase angle
-        wires (int): the subsystem the gate acts on
-    """
-    num_params = 1
-    num_wires = 2
-    par_domain = "R"
-    grad_method = "A"
-
-    @staticmethod
-    def decomposition(phi, wires):
-        return [
-            qml.CNOT(wires=wires),
-            qml.RZ(phi, wires=[wires[1]]),
-            qml.CNOT(wires=wires),
-        ]
-
-    @classmethod
-    def _matrix(cls, *params):
-        phi = params[0]
-        pos_phase = np.exp(1.0j * phi / 2)
-        neg_phase = np.exp(-1.0j * phi / 2)
-        return np.diag([neg_phase, pos_phase, pos_phase, neg_phase])
